'use strict';

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _expect = require('expect');

var _expect2 = _interopRequireDefault(_expect);

var _mocks = require('../mocks');

var _index = require('./index');

var _index2 = _interopRequireDefault(_index);

var _stream = require('../stream');

var _stream2 = _interopRequireDefault(_stream);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* eslint-disable require-jsdoc */
/* eslint-env mocha */
describe('ConnectionGroup', function () {
  var group = void 0,
      conn = void 0;

  beforeEach(function () {
    group = new _index2.default();
    conn = new _mocks.Connection({ id: 'conn1' });
  });

  describe('static "ensure" method', function () {
    it('returns the value if it is a connection group', function () {
      var result = _index2.default.ensure(group);

      (0, _expect2.default)(result).toBe(group);
    });

    it('wraps connections into a connection group', function () {
      var result = _index2.default.ensure(conn);

      (0, _expect2.default)(result).toBeA(_index2.default);
      (0, _expect2.default)([].concat((0, _toConsumableArray3.default)(result))).toContain(conn);
    });
  });

  describe('id()', function () {
    it('returns null if no connection is found', function () {
      var result = group.id('no such connection');

      (0, _expect2.default)(result).toBe(null);
    });

    it('returns the matching connection', function () {
      group.add(conn);
      var result = group.id(conn.id);

      (0, _expect2.default)(result).toBe(conn);
    });
  });

  describe('add()', function () {
    it('adds the connection', function () {
      group.add(conn);

      (0, _expect2.default)(group.id(conn.id)).toBe(conn);
    });

    it('emits "add"', function () {
      var spy = (0, _expect.createSpy)();
      group.on('add', spy);
      group.add(conn);

      (0, _expect2.default)(spy).toHaveBeenCalled();
      (0, _expect2.default)(spy).toHaveBeenCalledWith(conn);
    });

    it('does not emit "add" if already contained', function () {
      var spy = (0, _expect.createSpy)();
      group.on('add', spy);

      group.add(conn);
      group.add(conn);

      (0, _expect2.default)(spy.calls.length).toBe(1);
    });

    it('returns the connection', function () {
      var result = group.add(conn);

      (0, _expect2.default)(result).toBe(conn);
    });
  });

  describe('remove()', function () {
    it('removes connections from the group', function () {
      group.add(conn);
      group.remove(conn);

      (0, _expect2.default)(group.id(conn.id)).toBe(null);
    });

    it('emits "remove"', function () {
      var spy = (0, _expect.createSpy)();
      group.on('remove', spy);
      group.add(conn);
      group.remove(conn);

      (0, _expect2.default)(spy).toHaveBeenCalled();
      (0, _expect2.default)(spy).toHaveBeenCalledWith(conn);
    });

    it('does not emit "remove" twice', function () {
      var spy = (0, _expect.createSpy)();

      group.on('remove', spy);
      group.add(conn);
      group.remove(conn);
      group.remove(conn);

      (0, _expect2.default)(spy.calls.length).toBe(1);
    });

    it('does not explode if the connection does not exist', function () {
      (0, _expect2.default)(function () {
        return group.remove(conn);
      }).toNotThrow();
    });

    it('returns the connection', function () {
      var result = group.remove(conn);

      (0, _expect2.default)(result).toBe(conn);
    });
  });

  describe('has()', function () {
    it('returns true if the connection is contained', function () {
      var group = new _index2.default();
      group.add(conn);
      var isContained = group.has(conn);

      (0, _expect2.default)(isContained).toBe(true);
    });

    it('returns false if there is no such connection', function () {
      var group = new _index2.default();
      var isContained = group.has(conn);

      (0, _expect2.default)(isContained).toBe(false);
    });

    it('ignores inherited properties', function () {
      var conn = new _mocks.Connection({ id: 'toString' });
      var group = new _index2.default();
      var isContained = group.has(conn);

      (0, _expect2.default)(isContained).toBe(false);
    });
  });

  describe('iterator', function () {
    it('yields every connection', function () {
      var conn1 = new _mocks.Connection({ id: 'conn1' });
      var conn2 = new _mocks.Connection({ id: 'conn2' });

      group.add(conn1);
      group.add(conn2);

      var results = [].concat((0, _toConsumableArray3.default)(group));
      (0, _expect2.default)(results).toContain(conn1);
      (0, _expect2.default)(results).toContain(conn2);
      (0, _expect2.default)(results.length).toBe(2);
    });
  });

  describe('send()', function () {
    it('sends the message through each member', function () {
      conn.send = (0, _expect.createSpy)();
      group.add(conn);

      var msg = { isMessage: true };
      group.send(msg);

      (0, _expect2.default)(conn.send).toHaveBeenCalled();
      (0, _expect2.default)(conn.send).toHaveBeenCalledWith(msg);
    });
  });

  describe('filter()', function () {
    it('returns a new ConnectionGroup', function () {
      var result = group.filter(function () {
        return false;
      });

      (0, _expect2.default)(result).toBeA(_index2.default);
      (0, _expect2.default)(result).toNotBe(group);
    });

    it('only contains items that match the terms', function () {
      group.add(conn);
      group.add(new _mocks.Connection({ id: 'conn2' }));

      var result = group.filter(function (conn) {
        return conn.id === 'conn1';
      });

      (0, _expect2.default)([].concat((0, _toConsumableArray3.default)(result)).length).toBe(1);
    });
  });

  describe('type()', function () {
    it('returns a new group', function () {
      var result = group.type('connection-type');

      (0, _expect2.default)(result).toBeA(_index2.default);
      (0, _expect2.default)(result).toNotBe(group);
    });

    it('returns connections matching the type', function () {
      var conn = new _mocks.Connection({ id: 'conn1', type: 'websocket' });
      group.add(conn);

      group.add(new _mocks.Connection({ id: 'conn2', type: 'http' }));
      group.add(new _mocks.Connection({ id: 'conn3', type: 'http' }));

      var result = group.type('websocket');

      (0, _expect2.default)([].concat((0, _toConsumableArray3.default)(result))).toContain(conn);
      (0, _expect2.default)([].concat((0, _toConsumableArray3.default)(result)).length).toBe(1);
    });
  });

  describe('messages stream', function () {
    var spy = void 0;

    beforeEach(function () {
      spy = (0, _expect.createSpy)();
      group.add(conn);
    });

    it('is a stream', function () {
      (0, _expect2.default)(group.messages).toBeA(_stream2.default);
    });

    it('reports every message in the group', function () {
      group.messages.forEach(spy);
      conn.push('hello');

      (0, _expect2.default)(spy).toHaveBeenCalled();
      (0, _expect2.default)(spy).toHaveBeenCalledWith('hello');
    });

    it('reports messages from connections added later', function () {
      var conn = new _mocks.Connection({ id: 'conn2' });
      group.messages.forEach(spy);
      group.add(conn);
      conn.push('message');

      (0, _expect2.default)(spy).toHaveBeenCalled();
      (0, _expect2.default)(spy).toHaveBeenCalledWith('message');
    });

    it('does not report messages from removed connections', function () {
      group.messages.forEach(spy);
      group.remove(conn);
      conn.push('message');

      (0, _expect2.default)(spy).toNotHaveBeenCalled();
    });

    it('does not report messages from added, then removed connections', function () {
      var conn = new _mocks.Connection({ id: 'conn2' });
      group.messages.forEach(spy);
      group.add(conn);
      group.remove(conn);
      conn.push('message');

      (0, _expect2.default)(spy).toNotHaveBeenCalled();
    });

    it('disposes of each subscription when dispose is called', function () {
      var dispose = (0, _expect.createSpy)();
      conn.messages.forEach = (0, _expect.createSpy)().andReturn({ dispose: dispose });

      group.add(conn);
      group.messages.forEach(spy).dispose();

      (0, _expect2.default)(dispose).toHaveBeenCalled();
    });

    it('unsubscribes on dispose', function () {
      group.messages.forEach(function () {}).dispose();

      (0, _expect2.default)(group.listeners('add')).toEqual([]);
      (0, _expect2.default)(group.listeners('remove')).toEqual([]);
    });
  });

  describe('"union" method', function () {
    var group1 = void 0,
        group2 = void 0,
        conn1 = void 0,
        conn2 = void 0;

    beforeEach(function () {
      group1 = new _index2.default();
      group2 = new _index2.default();

      conn1 = new _mocks.Connection({ id: 'conn1' });
      conn2 = new _mocks.Connection({ id: 'conn2' });
    });

    it('creates a new group', function () {
      var group = group1.union(group2);

      (0, _expect2.default)(group).toBeA(_index2.default);
      (0, _expect2.default)(group).toNotBe(group1);
      (0, _expect2.default)(group).toNotBe(group2);
    });

    it('adds every item in both source groups', function () {
      group1.add(conn1);
      group2.add(conn2);

      var group = group1.union(group2);

      (0, _expect2.default)([].concat((0, _toConsumableArray3.default)(group))).toContain(conn1);
      (0, _expect2.default)([].concat((0, _toConsumableArray3.default)(group))).toContain(conn2);
    });

    it('adds connections as they arive', function () {
      var group = group1.union(group2);

      group1.add(conn1);
      group2.add(conn2);

      (0, _expect2.default)([].concat((0, _toConsumableArray3.default)(group))).toContain(conn1);
      (0, _expect2.default)([].concat((0, _toConsumableArray3.default)(group))).toContain(conn2);
    });

    it('removes a connection if neither group has it', function () {
      group1.add(conn1);
      group2.add(conn1);

      group2.add(conn2);

      var group = group1.union(group2);

      group1.remove(conn1);
      group2.remove(conn2);

      (0, _expect2.default)([].concat((0, _toConsumableArray3.default)(group))).toContain(conn1);
      (0, _expect2.default)([].concat((0, _toConsumableArray3.default)(group))).toNotContain(conn2);
    });
  });
});